import os
from typing import Iterator
from unittest import mock

import pytest

from pyfakefs.fake_filesystem import FakeFilesystem

from gcs_migration.parsers import CephConfig


@pytest.fixture
def ceph_host_name() -> str:
    return "ceph.example.org"


@pytest.fixture
def ceph_endpoint(ceph_host_name: str, ceph_use_https: bool) -> str:
    return f"{'https' if ceph_use_https else 'http'}://{ceph_host_name}"


@pytest.fixture
def ceph_rgw_admin_access_key_id() -> str:
    return "admin_access_key_id"


@pytest.fixture
def ceph_rgw_admin_secret_access_key() -> str:
    return "admin_secret_access_key"


@pytest.fixture
def ceph_rgw_admin_bucket() -> str:
    return "admin"


@pytest.fixture(
    params=[
        True,
        pytest.param(False, marks=pytest.mark.connector_only("ceph")),
    ],
    ids=["ceph_https", "ceph_http"],
)
def ceph_use_https(request: pytest.FixtureRequest) -> bool:
    return request.param


@pytest.fixture
def ceph_config(
    ceph_host_name: str,
    ceph_use_https: bool,
    ceph_rgw_admin_access_key_id: str,
    ceph_rgw_admin_secret_access_key,
    ceph_rgw_admin_bucket: str,
) -> str:
    return f"""\
# Fully-qualified domain name of the Ceph Rados Gateway S3-compatible service,
# with a port number optionally appended (e.g radosgw.example.com or radosgw.example.com:8443)
host_name = {ceph_host_name}

# Path to a mime-types file to associate mime types with extensions for
# objects stored in this service (no default)
# mime_types_path = /etc/mime.types

# Set to true to use https, or false to use http (default to https)
use_https = {'true' if ceph_use_https else 'false'}

# Use virtual-host style (bucket as host name) for https requests
# (default to false)
use_virtual_host = false

# Verify https peer credentials
# This requires that the rgw S3 service's ssl certificate be signed by a CA
# included in the operating system's trust root.  To add trusted CA certs,
# see the man page of update-ca-certificates or update-ca-trust, depending
# on the system.
# (default to true)
verify_peer = true

# Rados Gateway Admin bucket (default to admin)
ceph_rgw_admin_bucket = {ceph_rgw_admin_bucket}

# AccessKeyId for a Rados Gateway User with users:read caps
ceph_rgw_admin_access_key_id = {ceph_rgw_admin_access_key_id}

# SecretAccessKey for a Rados Gateway User with users:read caps
ceph_rgw_admin_secret_access_key = {ceph_rgw_admin_secret_access_key}

# Maximum number of retries for connection faults
max_retries = 5

# Base retry delay for exponential back-off when retrying (in ms)
retry_delay = 400
"""


@pytest.fixture
def mock_ceph_config(
    ceph_host_name: str,
    ceph_use_https: str,
    ceph_rgw_admin_access_key_id: str,
    ceph_rgw_admin_secret_access_key,
    ceph_rgw_admin_bucket: str,
) -> CephConfig:
    res = mock.MagicMock(spec=CephConfig)
    res.path = "/etc/globus/globus-gridftp-server-ceph.conf"
    res.host_name = ceph_host_name
    res.use_https = ceph_use_https
    res.ceph_rgw_admin_bucket = ceph_rgw_admin_bucket
    res.ceph_rgw_admin_access_key_id = ceph_rgw_admin_access_key_id
    res.ceph_rgw_admin_secret_access_key = ceph_rgw_admin_secret_access_key

    return res


@pytest.fixture
def ceph_config_file(fs: FakeFilesystem, ceph_config: str) -> Iterator[str]:
    ceph_config_file_path = "/etc/globus/globus-gridftp-server-ceph.conf"
    fs.makedirs(os.path.dirname(ceph_config_file_path))
    with open(ceph_config_file_path, "w") as f:
        f.write(ceph_config)
    yield ceph_config_file_path


@pytest.fixture
def ceph_config_object(fs: FakeFilesystem, ceph_config_file: str) -> CephConfig:
    return CephConfig(ceph_config_file)
